@@ -31,6 +31,19 @@ class UpdateExpression(object):
     def __init__(self, query):
         self.query = query
         self.update_data = {}
+        self.__upsert = False
+        self.__multi = False
+    
+    def upsert(self):
+        ''' If a document matching the query doesn't exist, create one '''
+        self.__upsert = True
+        return self
+    
+    def multi(self):
+        ''' Update multiple documents.  The Mongo default is to only update 
+            the first matching document '''
+        self.__multi = True
+        return self
     
     def set(self, qfield, value):
         ''' Atomically set ``qfield`` to ``value``'''
@@ -84,6 +97,8 @@ class UpdateExpression(object):
         return self._atomic_generic_op('$pop', qfield, -1)
     
     def _atomic_list_op_multivalue(self, op, qfield, *value):
+        if op not in qfield.get_type().valid_modifiers:
+            raise InvalidModifierException(qfield.get_type(), op)
         wrapped = []
         for v in value:
             wrapped.append(qfield.get_type().item_type.wrap(v))
@@ -93,18 +108,27 @@ class UpdateExpression(object):
         return self
     
     def _atomic_list_op(self, op, qfield, value):
+        if op not in qfield.get_type().valid_modifiers:
+            raise InvalidModifierException(qfield.get_type(), op)
+        
         if op not in self.update_data:
             self.update_data[op] = {}
         self.update_data[op][qfield.get_name()] = qfield.get_type().child_type().wrap(value)
         return self
     
     def _atomic_op(self, op, qfield, value):
+        if op not in qfield.get_type().valid_modifiers:
+            raise InvalidModifierException(qfield.get_type(), op)
+
         if op not in self.update_data:
             self.update_data[op] = {}
         self.update_data[op][qfield.get_name()] = qfield.get_type().wrap(value)
         return self
     
     def _atomic_generic_op(self, op, qfield, value):
+        if op not in qfield.get_type().valid_modifiers:
+            raise InvalidModifierException(qfield.get_type(), op)
+        
         if op not in self.update_data:
             self.update_data[op] = {}
         self.update_data[op][qfield.get_name()] = value
@@ -116,5 +140,18 @@ class UpdateExpression(object):
         collection = self.query.db[self.query.type.get_collection_name()]
         for index in self.query.type.get_indexes():
             index.ensure(collection)
-        collection.update(self.query.query, self.update_data)
+        collection.update(self.query.query, self.update_data, upsert=self.__upsert, multi=self.__multi)
 
+class UpdateException(Exception):
+    ''' Base class for exceptions related to updates '''
+    pass
+
+class InvalidModifierException(UpdateException):
+    ''' Exception raised if a modifier was used which isn't valid for a field '''
+    def __init__(self, field, op):
+        UpdateException.__init__(self, 'Invalid modifier for %s field: %s' % (field.__class__.__name__, op))
+
+class ConflictingModifierException(UpdateException):
+    ''' Exception raised if conflicting modifiers are being used in the 
+        update expression '''
+    pass
\ No newline at end of file
